module CortexM1DbgIntegrationWrapper (
  /*AUTOARG*/
  // Outputs
  DBGRESTARTED, JTAGNSW, JTAGTOP, TDO, nTDOEN, SWDO, SWDOEN, LOCKUP, 
  HALTED, SYSRESETREQ, HTRANS, HBURST, HPROT, HSIZE, HWRITE, 
  HMASTLOCK, HADDR, HWDATA, 
  // Inputs
  HCLK, SYSRESETn, IRQ, NMI, DBGRESETn, EDBGRQ, DBGRESTART, nTRST, 
  SWCLKTCK, SWDITMS, TDI, HREADY, HRESP, HRDATA
  );

  // Clocks and Reset
  input                 HCLK;          // System clock
  input                 SYSRESETn;     // System reset
  input                 DBGRESETn;     // Power on debug reset

  // Misc status signals
  output                LOCKUP;        // Core is in lockup state
  output                HALTED;        // Core is in Halt debug state

  // JTAG and serial wire interface
  input                 nTRST;         // JTAG reset
  input                 SWCLKTCK;      // Serial wire and JTAG clock
  input                 SWDITMS;       // SW Data / JTAG Test Mode Select
  input                 TDI;           // JTAG data input
  output                JTAGNSW;       // JTAG = 1, serial wire = 0
  output                JTAGTOP;       // state controller indicator
  output                TDO;           // JTAG data output
  output                nTDOEN;        // JTAG data out enable
  output                SWDO;          // Serial wire data out
  output                SWDOEN;        // Serial data output enable

  // AHB-Lite port
  input                 HREADY;        // Slave ready signal
  input                 HRESP;         // Slave response signal
  input [31:0]          HRDATA;        // Data from slave to master
  output [1:0]          HTRANS;        // Transfer type
  output [2:0]          HBURST;        // Burst type
  output [3:0]          HPROT;         // Transfer protection bits
  output [2:0]          HSIZE;         // Transfer size
  output                HWRITE;        // Transfer direction
  output                HMASTLOCK;     // Transfer is a locked transfer
  output [31:0]         HADDR;         // Transfer address
  output [31:0]         HWDATA;        // Data from master to slave

  // NVIC
  input [31:0]          IRQ;           // External interrupts
  input                 NMI;           // Non-maskable interrupt
  output                SYSRESETREQ;   // From u_cortexm1 of CortexM1Dbg.v
  
  // Debug halt/restart signals
  input                 EDBGRQ;        // External debug request
  input                 DBGRESTART;    // Restart from halt request
  output                DBGRESTARTED;  // Restart from halt acknowledgei

CortexM1DbgIntegration u_CortexM1DbgIntegration_inst(
    .DBGRESTARTED(DBGRESTARTED),
    .JTAGNSW(JTAGNSW),
    .JTAGTOP(JTAGTOP), 
    .TDO(TDO), 
    .nTDOEN(nTDOEN), 
    .SWDO(SWDO), 
    .SWDOEN(SWDOEN), 
    .LOCKUP(LOCKUP), 
    .HALTED(HALTED),
    .SYSRESETREQ(SYSRESETREQ),
    .HTRANS(HTRANS), 
    .HBURST(HBURST), 
    .HPROT(HPROT), 
    .HSIZE(HSIZE), 
    .HWRITE(HWRITE), 
    .HMASTLOCK(HMASTLOCK), 
    .HADDR(HADDR), 
    .HWDATA(HWDATA), 
  // Inputs
    .HCLK(HCLK), 
    .SYSRESETn(SYSRESETn), 
    .IRQ(IRQ), 
    .NMI(NMI),
    .DBGRESETn(DBGRESETn),
    .EDBGRQ(EDBGRQ),
    .DBGRESTART(DBGRESTART),
    .nTRST(nTRST), 
    .SWCLKTCK(SWCLKTCK), 
    .SWDITMS(SWDITMS), 
    .TDI(TDI), 
    .HREADY(HREADY), 
    .HRESP(HRESP), 
    .HRDATA(HRDATA)
);
endmodule
