//-----------------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM limited.
//
//            (C) COPYRIGHT 2007-2008 ARM limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM limited.
//
//      SVN Information
//
//      Checked In          : $Date: 2008-04-17 12:26:16 +0100 (Thu, 17 Apr 2008) $
//
//      Revision            : $Revision: 80952 $
//
//      Release Information : Cortex-M1-AT470-r1p0-00rel0
//-----------------------------------------------------------------------------

module cm1_undef_check (OPCODE, SECOND, DEBUG_ENABLED, UNDEFINED, UNPREDICTABLE);

   parameter  OS    = 1;
  
   input [15:0] OPCODE;
   input [15:0] SECOND;
   input        DEBUG_ENABLED;
   output       UNDEFINED;
   output       UNPREDICTABLE;

   // --------------------------------------------------------------
   // UAL instructions ADC through ASR
   // --------------------------------------------------------------

   wire         is_adc_reg_t1     = (OPCODE[15: 6] == 'b0100000101);
   wire         is_add_imm_t1     = (OPCODE[15: 9] == 'b0001110);
   wire         is_add_imm_t2     = (OPCODE[15:11] == 'b00110);
   wire         is_add_reg_t1     = (OPCODE[15: 9] == 'b0001100);
   wire         is_add_reg_t2     = (OPCODE[15: 8] == 'b01000100);
   wire         is_add_spi_t1     = (OPCODE[15:11] == 'b10101);
   wire         is_add_spi_t2     = (OPCODE[15: 7] == 'b101100000);
   wire         is_add_spr_t1     = (OPCODE[15: 8] == 'b01000100)
                                  & (OPCODE[ 6: 3] == 'b1101);
   wire         is_add_spr_t2     = (OPCODE[15: 7] == 'b010001001)
                                  & (OPCODE[ 2: 0] == 'b101);
   wire         is_adr_t1         = (OPCODE[15:11] == 'b10100);
   wire         is_and_reg_t1     = (OPCODE[15: 6] == 'b0100000000);
   wire         is_asr_imm_t1     = (OPCODE[15:11] == 'b00010);
   wire         is_asr_reg_t1     = (OPCODE[15: 6] == 'b0100000100);

   wire         sect_AA_def       = is_adc_reg_t1  | is_add_imm_t1
                                  | is_add_imm_t2  | is_add_reg_t1
                                  | is_add_reg_t2  | is_add_spi_t1
                                  | is_add_spi_t2  | is_add_spr_t1
                                  | is_add_spr_t2  | is_adr_t1
                                  | is_and_reg_t1  | is_asr_imm_t1
                                  | is_asr_reg_t1;

   wire         sect_AA_unp       = 1'b0;

   // --------------------------------------------------------------
   // UAL instructions B through ISB
   // --------------------------------------------------------------


   wire         is_b_t1           = (OPCODE[15:12] == 'b1101) 
                                  & (OPCODE[11: 8] != 'b1110) // UND
                                  & (OPCODE[11: 8] != 'b1111);// SVC
  
   wire         is_b_t2           = (OPCODE[15:11] == 'b11100);

  
   wire         is_bic_t1         = (OPCODE[15: 6] == 'b0100001110);
   wire         is_bkpt_t1        = (OPCODE[15: 8] == 'b10111110)
                                  & DEBUG_ENABLED;

   wire         is_bl_t1          = (SECOND[15:14] == 'b11)
                                  & (SECOND[12:12] == 'b1)
                                  & (OPCODE[15:11] == 'b11110);
  
   wire         is_blx_t1         = (OPCODE[15: 7] == 'b010001111);
   wire         is_blx_t1_unp     = (OPCODE[15: 7] == 'b010001111)
                                  & ( (OPCODE[ 6: 3] == 'b1111)
                                    | (OPCODE[ 2: 0] != 'b000)
                                    );
   
   wire         is_bx_t1          = (OPCODE[15: 7] == 'b010001110);
   wire         is_bx_t1_unp      = (OPCODE[15: 7] == 'b010001110)
                                  & (OPCODE[ 2: 0] != 'b0000);
  
   wire         is_cmn_t1         = (OPCODE[15: 6] == 'b0100001011);
   wire         is_cmp_imm_t1     = (OPCODE[15:11] == 'b00101);
   wire         is_cmp_reg_t1     = (OPCODE[15: 6] == 'b0100001010);
   wire         is_cmp_reg_t2     = (OPCODE[15: 8] == 'b01000101);
   wire         is_cmp_reg_t2_unp = (OPCODE[15: 8] == 'b01000101)
                                  & (OPCODE[ 7: 7] == 'b0)
                                  & (OPCODE[ 6: 6] == 'b0);
   
   wire         is_cps_t1         = (OPCODE[15: 5] == 'b10110110011);
   wire         is_cps_t1_unp     = (OPCODE[15: 5] == 'b10110110011)
                                  & (OPCODE[ 3: 2] != 'b00);
   
   wire         is_cpy_t1         = (1'b0); // deprecated


  wire          is_dmb_t1         = (OPCODE[15: 4] == 'b111100111011)
                                  & (SECOND[15:14] == 'b10)
                                  & (SECOND[12:12] == 'b0)  
                                  & (SECOND[ 7: 4] == 'b0101);

  wire          is_dmb_t1_unp     = (OPCODE[15: 4] == 'b111100111011)
                                  & (SECOND[15:14] == 'b10)
                                  & (SECOND[12:12] == 'b0)  
                                  & (SECOND[ 7: 4] == 'b0101)
                                  & ( (OPCODE[ 3: 0] != 'b1111)
                                    | (SECOND[11: 8] != 'b1111)
                                    | (SECOND[13:13] != 'b0)  
                                    );

  wire          is_dsb_t1         = (OPCODE[15: 4] == 'b111100111011)
                                  & (SECOND[15:14] == 'b10)
                                  & (SECOND[12:12] == 'b0)
                                  & (SECOND[ 7: 4] == 'b0100);
    
  wire          is_dsb_t1_unp     = (OPCODE[15: 4] == 'b111100111011)
                                  & (SECOND[15:14] == 'b10)
                                  & (SECOND[12:12] == 'b0)
                                  & (SECOND[ 7: 4] == 'b0100)
                                  & ( (OPCODE[ 3: 0] != 'b1111)
                                    | (SECOND[11: 8] != 'b1111)
                                    | (SECOND[13:13] != 'b0)  
                                    );
  
   wire         is_eor_t1         = (OPCODE[15: 6] == 'b0100000001);  

   wire         is_isb_t1         = (OPCODE[15: 4] == 'b111100111011)
                                  & (SECOND[15:14] == 'b10)
                                  & (SECOND[12:12] == 'b0)
                                  & (SECOND[ 7: 4] == 'b0110);

   wire         is_isb_t1_unp     = (OPCODE[15: 4] == 'b111100111011)
                                  & (SECOND[15:14] == 'b10)
                                  & (SECOND[12:12] == 'b0)
                                  & (SECOND[ 7: 4] == 'b0110)
                                  & ( (OPCODE[ 3: 0] != 'b1111)
                                    | (SECOND[11: 8] != 'b1111)
                                    | (SECOND[13:13] != 'b0)  
                                    );
   
   wire         sect_BI_def       = is_b_t1        | is_b_t2
                                  | is_bic_t1      | is_bkpt_t1
                                  | is_bl_t1       | is_blx_t1
                                  | is_bx_t1       | is_cmn_t1
                                  | is_cmp_imm_t1  | is_cmp_reg_t1
                                  | is_cmp_reg_t2  | is_cps_t1 
                                  | is_cpy_t1      | is_dmb_t1
                                  | is_dsb_t1      | is_eor_t1
                                  | is_isb_t1;

   wire         sect_BI_unp       = is_bx_t1_unp
                                  | is_blx_t1_unp 
                                  | is_cmp_reg_t2_unp
                                  | is_cps_t1_unp
                                  | is_dmb_t1_unp
                                  | is_dsb_t1_unp
                                  | is_isb_t1_unp;
   
   // --------------------------------------------------------------
   // UAL instructions LDM through LSR
   // --------------------------------------------------------------

   wire         is_ldm_t1         = (OPCODE[15:11] == 'b11001);
   wire         is_ldm_t1_unp     = (OPCODE[15:11] == 'b11001)
                                  & (OPCODE[ 7: 0] == 'b00000000);
   wire         is_ldr_imm_t1     = (OPCODE[15:11] == 'b01101);
   wire         is_ldr_imm_t2     = (OPCODE[15:11] == 'b10011);
   wire         is_ldr_lit_t1     = (OPCODE[15:11] == 'b01001);
   wire         is_ldr_reg_t1     = (OPCODE[15: 9] == 'b0101100);
   wire         is_ldrb_imm_t1    = (OPCODE[15:11] == 'b01111);
   wire         is_ldrb_reg_t1    = (OPCODE[15: 9] == 'b0101110);
   wire         is_ldrh_imm_t1    = (OPCODE[15:11] == 'b10001);
   wire         is_ldrh_reg_t1    = (OPCODE[15: 9] == 'b0101101);
   wire         is_ldrsb_t1       = (OPCODE[15: 9] == 'b0101011);
   wire         is_ldrsh_t1       = (OPCODE[15: 9] == 'b0101111);
   wire         is_lsl_imm_t1     = (OPCODE[15:11] == 'b00000);
   wire         is_lsl_reg_t1     = (OPCODE[15: 6] == 'b0100000010);
   wire         is_lsr_imm_t1     = (OPCODE[15:11] == 'b00001);
   wire         is_lsr_reg_t1     = (OPCODE[15: 6] == 'b0100000011);

   wire         sect_LL_def       = is_ldm_t1      | is_ldr_imm_t1
                                  | is_ldr_imm_t2  | is_ldr_lit_t1 
                                  | is_ldr_reg_t1  | is_ldrb_imm_t1 
                                  | is_ldrb_reg_t1 | is_ldrh_imm_t1
                                  | is_ldrh_reg_t1 | is_ldrsb_t1 
                                  | is_ldrsh_t1    | is_lsl_imm_t1
                                  | is_lsl_reg_t1  | is_lsr_imm_t1
                                  | is_lsr_reg_t1;

   wire         sect_LL_unp       = is_ldm_t1_unp;

   // --------------------------------------------------------------
   // UAL instructions MOV through MVN
   // --------------------------------------------------------------

   wire         is_mov_imm_t1     = (OPCODE[15:11] == 'b00100);
   wire         is_mov_reg_t1     = (OPCODE[15: 8] == 'b01000110);
   wire         is_mov_reg_t2     = (OPCODE[15: 6] == 'b0000000000);
   wire         is_mov_shf_t1     = (1'b0); // see ASR/LSL/LSR/ROR
   wire         is_mrs_t1         = (OPCODE[15: 5] == 'b11110011111)
                                  & (SECOND[15:14] == 'b10)
                                  & (SECOND[12:12] == 'b0);

  wire          is_mrs_t1_unp     = (OPCODE[15: 5] == 'b11110011111)
                                  & (SECOND[15:14] == 'b10)
                                  & (SECOND[12:12] == 'b0)
                                  & ( (SECOND[11: 8] == 'b1101)
                                    | (SECOND[11: 8] == 'b1111)
                                    | (SECOND[ 7: 0] >= 8'd21)
                                    | (SECOND[ 7: 0] == 8'd4)
                                    | (SECOND[ 7: 0] == 8'd10)
                                    | (SECOND[ 7: 0] == 8'd11)
                                    | (SECOND[ 7: 0] == 8'd12)
                                    | (SECOND[ 7: 0] == 8'd13)
                                    | (SECOND[ 7: 0] == 8'd14)
                                    | (SECOND[ 7: 0] == 8'd15)
                                    | (SECOND[ 7: 0] == 8'd17)
                                    | (SECOND[ 7: 0] == 8'd18)
                                    | (SECOND[ 7: 0] == 8'd19)
                                    | (OPCODE[ 4: 0] != 'b01111)
                                    | (SECOND[13:13] != 'b0)
                                    );
  
   wire         is_msr_t1         = (OPCODE[15: 5] == 'b11110011100)
                                  & (SECOND[15:14] == 'b10)
                                  & (SECOND[12: 8] == 'b01000);

  wire          is_msr_t1_unp     = (OPCODE[15: 5] == 'b11110011100)
                                  & (SECOND[15:14] == 'b10)
                                  & (SECOND[12: 8] == 'b01000)
                                  & ( (OPCODE[ 3: 0] == 'b1101)
                                    | (OPCODE[ 3: 0] == 'b1111)
                                    | (SECOND[ 7: 0] >= 8'd21)
                                    | (SECOND[ 7: 0] == 8'd4)
                                    | (SECOND[ 7: 0] == 8'd10)
                                    | (SECOND[ 7: 0] == 8'd11)
                                    | (SECOND[ 7: 0] == 8'd12)
                                    | (SECOND[ 7: 0] == 8'd13)
                                    | (SECOND[ 7: 0] == 8'd14)
                                    | (SECOND[ 7: 0] == 8'd15)
                                    | (SECOND[ 7: 0] == 8'd17)
                                    | (SECOND[ 7: 0] == 8'd18)
                                    | (SECOND[ 7: 0] == 8'd19)
                                    | (OPCODE[ 4: 4] != 'b0)
                                    | (SECOND[13:13] != 'b0)
                                    );

  
   wire         is_mul_t1         = (OPCODE[15: 6] == 'b0100001101);
   wire         is_mvn_t1         = (OPCODE[15: 6] == 'b0100001111);

   wire         sect_MM_def       = is_mov_imm_t1  | is_mov_reg_t1
                                  | is_mov_reg_t2  | is_mov_shf_t1 
                                  | is_mrs_t1      | is_msr_t1
                                  | is_mul_t1      | is_mvn_t1;

   wire         sect_MM_unp       = is_mrs_t1_unp
                                  | is_msr_t1_unp;
   
   // --------------------------------------------------------------
   // UAL instructions NEG through RSB
   // --------------------------------------------------------------

   wire         is_neg_t1         = (1'b0); // now RSBS
   wire         is_nop_t1         = (OPCODE[15: 8] == 'b10111111)
                                  & (OPCODE[ 7: 0] == 'b00000000);

   // NOP compatible hints encompassing NOP/YIELD/SEV/WFE etc.
   wire         is_nop_c_t1       = (OPCODE[15: 8] == 'b10111111)
                                  & (OPCODE[ 3: 0] == 'b0000);

  
   wire         is_orr_t1         = (OPCODE[15: 6] == 'b0100001100);
   wire         is_pop_t1         = (OPCODE[15: 9] == 'b1011110);
   wire         is_pop_t1_unp     = (OPCODE[15: 9] == 'b1011110)
                                  & (OPCODE[ 8: 0] == 'b000000000);
   wire         is_push_t1        = (OPCODE[15: 9] == 'b1011010);
   wire         is_push_t1_unp    = (OPCODE[15: 9] == 'b1011010)
                                  & (OPCODE[ 8: 0] == 'b000000000);
   wire         is_rev_t1         = (OPCODE[15: 6] == 'b1011101000);
   wire         is_rev16_t1       = (OPCODE[15: 6] == 'b1011101001);
   wire         is_revsh_t1       = (OPCODE[15: 6] == 'b1011101011);
   wire         is_ror_t1         = (OPCODE[15: 6] == 'b0100000111);
   wire         is_rsb_t1         = (OPCODE[15: 6] == 'b0100001001);

   wire         sect_NR_def       = is_neg_t1      | is_nop_t1
                                  | is_nop_c_t1 
                                  | is_orr_t1      | is_pop_t1
                                  | is_push_t1     | is_rev_t1
                                  | is_rev16_t1    | is_revsh_t1
                                  | is_ror_t1      | is_rsb_t1;

   wire         sect_NR_unp       = is_pop_t1_unp
                                  | is_push_t1_unp;
   
   // --------------------------------------------------------------
   // UAL instructions SBC through SXTH
   // --------------------------------------------------------------

   wire         is_sbc_t1         = (OPCODE[15: 6] == 'b0100000110);
   wire         is_sev_t1         = (OPCODE[15: 8] == 'b10111111)
                                  & (OPCODE[ 7: 0] == 'b01000000);
   wire         is_stm_t1         = (OPCODE[15:11] == 'b11000);
   wire         is_stm_t1_unp     = (OPCODE[15:11] == 'b11000)
                                  & (OPCODE[ 7: 0] == 'b00000000);
   wire         is_str_imm_t1     = (OPCODE[15:11] == 'b01100);
   wire         is_str_imm_t2     = (OPCODE[15:11] == 'b10010);
   wire         is_str_reg_t1     = (OPCODE[15: 9] == 'b0101000);
   wire         is_strb_imm_t1    = (OPCODE[15:11] == 'b01110);
   wire         is_strb_reg_t1    = (OPCODE[15: 9] == 'b0101010);
   wire         is_strh_imm_t1    = (OPCODE[15:11] == 'b10000);
   wire         is_strh_reg_t1    = (OPCODE[15: 9] == 'b0101001);
   wire         is_sub_imm_t1     = (OPCODE[15: 9] == 'b0001111);
   wire         is_sub_imm_t2     = (OPCODE[15:11] == 'b00111);
   wire         is_sub_reg_t1     = (OPCODE[15: 9] == 'b0001101);
   wire         is_sub_spi_t1     = (OPCODE[15: 7] == 'b101100001);
   wire         is_svc_t1         = (OPCODE[15: 8] == 'b11011111) &
                                    (OS != 0);
   wire         is_sxtb_t1        = (OPCODE[15: 6] == 'b1011001001);
   wire         is_sxth_t1        = (OPCODE[15: 6] == 'b1011001000);

   wire         sect_SS_def       = is_sbc_t1      | is_sev_t1
                                  | is_stm_t1      | is_str_imm_t1
                                  | is_str_imm_t2  | is_str_reg_t1
                                  | is_strb_imm_t1 | is_strb_reg_t1
                                  | is_strh_imm_t1 | is_strh_reg_t1
                                  | is_sub_imm_t1  | is_sub_imm_t2 
                                  | is_sub_reg_t1  | is_sub_spi_t1
                                  | is_svc_t1      | is_sxtb_t1
                                  | is_sxth_t1;

   wire         sect_SS_unp       = is_stm_t1_unp;
  

   // --------------------------------------------------------------
   // UAL instructions TST through YIELD
   // --------------------------------------------------------------

   wire         is_tst_t1         = (OPCODE[15: 6] == 'b0100001000);
   wire         is_uxtb_t1        = (OPCODE[15: 6] == 'b1011001011);
   wire         is_uxth_t1        = (OPCODE[15: 6] == 'b1011001010);
   wire         is_wfe_t1         = (OPCODE[15: 8] == 'b10111111)
                                  & (OPCODE[ 7: 0] == 'b00100000);
   wire         is_wfi_t1         = (OPCODE[15: 8] == 'b10111111)
                                  & (OPCODE[ 7: 0] == 'b00110000);
   wire         is_yield_t1       = (OPCODE[15: 8] == 'b10111111)
                                  & (OPCODE[ 7: 0] == 'b00010000);

   wire         sect_TY_def       = is_tst_t1      | is_uxtb_t1
                                  | is_uxth_t1     | is_wfe_t1
                                  | is_wfi_t1      | is_yield_t1;

   wire         sect_TY_unp       = 1'b0;

   // --------------------------------------------------------------
   // Generate outputs
   // --------------------------------------------------------------

   wire         defined           = sect_AA_def 
                                  | sect_BI_def
                                  | sect_LL_def
                                  | sect_MM_def
                                  | sect_NR_def
                                  | sect_SS_def
                                  | sect_TY_def;
   
   wire         unpred            = sect_AA_unp 
                                  | sect_BI_unp
                                  | sect_LL_unp
                                  | sect_MM_unp
                                  | sect_NR_unp
                                  | sect_SS_unp
                                  | sect_TY_unp;
   
   wire         UNDEFINED         = unpred ? 1'bX : ~defined;
   wire         UNPREDICTABLE     = unpred;

endmodule

// -----------------------------------------------------------------
// EOF
// -----------------------------------------------------------------
